/****************************************************************************/
/** G-FIT V1.0                                                             **/
/** Fitting hyper-Erlang distributions to trace data with the EM algorithm **/
/**                                                                        **/
/** Developed by Axel Thuemmler, July/August 2004                          **/
/** Copyright (c) 2004 University of Dortmund. All rights reserved.        **/
/**                                                                        **/
/** The G-FIT software package is distributed in open-source under the     **/
/** terms of the GNU general public license agreement. To document the     **/
/** usage of G-FIT, we ask each user to cite our scientific publications   **/
/** mentioned on our Web site.                                             **/
/****************************************************************************/
#ifndef GERLANGDIST_H
#define GERLANGDIST_H

#include "trace.h"


// Maximal number of Erlang phases
#define MAX_FACULTY	200


//---------------------
// Definition of class
//---------------------
class TGerlangDist
{

	public:

		// Constructor for hyper-Erlang distribution
		TGerlangDist(unsigned int _m);

		// Destructor
		~TGerlangDist();

		// Initialize the distribution parameters
		void init(unsigned int* _r, double* _lambdas = NULL , double* _weights = NULL);

		void adjust_m(unsigned int m_new);

		void get(unsigned int* _r, double* _lambdas, double* _weights);

		void print_r();

		void print_lambdas();

		void print_weights();

		// Evaluate Erlang pdf
		double erlang_pdf(unsigned int , double , double );

		// Evaluate Erlang cdf
		double erlang_cdf(unsigned int , double , double );

		double compute_loglikelihood(TTrace* );

		// Compute statistics of hyper-Erlang distribution
		void compute_distribution(const char* filename, double width, unsigned int steps);
		void compute_statistics(TTraceStatistics ts);
		void write_file(const char* filename);
		unsigned int read_file(const char* filename);

	private:
	
		// Initialize pre-computed faculty values
		void init_faculties();

		// Arrays with faculty values
		double fac[MAX_FACULTY+1];
		double lnfac[MAX_FACULTY+1];

		// number of different Erlang branches
		unsigned int m;

		// number of phases of i-th Erlang branch, i = 1,...,m
		unsigned int* r;		

		// lambda values of i-th Erlang branch, i = 1,...,m
		double* lambdas;

		// initial probabilities of i-th Erlang branch, i = 1,...,m
		double* weights;

	friend class TGerlangFit;

};

#endif
